@@ -1,3 +1,4 @@
+
 # Copyright 2012-2017 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,13 +14,14 @@
 # limitations under the License.
 
 import pickle, os, uuid
+import sys
 from pathlib import PurePath
 from collections import OrderedDict
 from .mesonlib import MesonException, commonpath
 from .mesonlib import default_libdir, default_libexecdir, default_prefix
 import ast
 
-version = '0.42.0'
+version = '0.44.0'
 backendlist = ['ninja', 'vs', 'vs2010', 'vs2015', 'vs2017', 'xcode']
 
 class UserOption:
@@ -29,9 +31,6 @@ class UserOption:
         self.choices = choices
         self.description = description
 
-    def parse_string(self, valuestring):
-        return valuestring
-
     # Check that the input is a valid value and return the
     # "cleaned" or "native" version. For example the Boolean
     # option could take the string "true" and return True.
@@ -72,13 +71,6 @@ class UserBooleanOption(UserOption):
     def set_value(self, newvalue):
         self.value = self.tobool(newvalue)
 
-    def parse_string(self, valuestring):
-        if valuestring == 'false':
-            return False
-        if valuestring == 'true':
-            return True
-        raise MesonException('Value "%s" for boolean option "%s" is not a boolean.' % (valuestring, self.name))
-
     def __bool__(self):
         return self.value
 
@@ -109,9 +101,6 @@ class UserIntegerOption(UserOption):
         except:
             raise MesonException('Value string "%s" is not convertable to an integer.' % valuestring)
 
-    def parse_string(self, valuestring):
-        return self.toint(valuestring)
-
     def validate_value(self, value):
         return self.toint(value)
 
@@ -136,27 +125,43 @@ class UserComboOption(UserOption):
             raise MesonException('Value %s not one of accepted values.' % value)
         return value
 
-class UserStringArrayOption(UserOption):
+class UserArrayOption(UserOption):
     def __init__(self, name, description, value, **kwargs):
         super().__init__(name, description, kwargs.get('choices', []))
-        self.set_value(value)
+        self.set_value(value, user_input=False)
 
-    def validate(self, value):
-        if isinstance(value, str):
-            if not value.startswith('['):
-                raise MesonException('Valuestring does not define an array: ' + value)
-            newvalue = ast.literal_eval(value)
+    def validate(self, value, user_input):
+        # User input is for options defined on the command line (via -D
+        # options). Users can put their input in as a comma separated
+        # string, but for defining options in meson_options.txt the format
+        # should match that of a combo
+        if not user_input:
+            if isinstance(value, str):
+                if not value.startswith('['):
+                    raise MesonException('Valuestring does not define an array: ' + value)
+                newvalue = ast.literal_eval(value)
+            else:
+                newvalue = value
         else:
-            newvalue = value
+            assert isinstance(value, str)
+            if value.startswith('['):
+                newvalue = ast.literal_eval(value)
+            else:
+                newvalue = [v.strip() for v in value.split(',')]
         if not isinstance(newvalue, list):
             raise MesonException('"{0}" should be a string array, but it is not'.format(str(newvalue)))
         for i in newvalue:
             if not isinstance(i, str):
                 raise MesonException('String array element "{0}" is not a string.'.format(str(newvalue)))
+        if self.choices:
+            bad = [x for x in newvalue if x not in self.choices]
+            if bad:
+                raise MesonException('Options "{}" are not in allowed choices: "{}"'.format(
+                    ', '.join(bad), ', '.join(self.choices)))
         return newvalue
 
-    def set_value(self, newvalue):
-        self.value = self.validate(newvalue)
+    def set_value(self, newvalue, user_input=True):
+        self.value = self.validate(newvalue, user_input)
 
     def validate_value(self, value):
         self.validate(value)
@@ -184,10 +189,7 @@ class CoreData:
         self.external_preprocess_args = {} # CPPFLAGS only
         self.external_args = {} # CPPFLAGS + CFLAGS
         self.external_link_args = {} # CFLAGS + LDFLAGS (with MSVC: only LDFLAGS)
-        if options.cross_file is not None:
-            self.cross_file = os.path.join(os.getcwd(), options.cross_file)
-        else:
-            self.cross_file = None
+        self.cross_file = self.__load_cross_file(options.cross_file)
         self.wrap_mode = options.wrap_mode
         self.compilers = OrderedDict()
         self.cross_compilers = OrderedDict()
@@ -196,6 +198,46 @@ class CoreData:
         # Only to print a warning if it changes between Meson invocations.
         self.pkgconf_envvar = os.environ.get('PKG_CONFIG_PATH', '')
 
+    @staticmethod
+    def __load_cross_file(filename):
+        """Try to load the cross file.
+
+        If the filename is None return None. If the filename is an absolute
+        (after resolving variables and ~), return that absolute path. Next,
+        check if the file is relative to the current source dir. If the path
+        still isn't resolved do the following:
+            Linux + BSD:
+                - $XDG_DATA_HOME/meson/cross (or ~/.local/share/meson/cross if
+                  undefined)
+                - $XDG_DATA_DIRS/meson/cross (or
+                  /usr/local/share/meson/cross:/usr/share/meson/cross if undefined)
+                - Error
+            *:
+                - Error
+        BSD follows the Linux path and will honor XDG_* if set. This simplifies
+        the implementation somewhat, especially since most BSD users wont set
+        those environment variables.
+        """
+        if filename is None:
+            return None
+        filename = os.path.expanduser(os.path.expandvars(filename))
+        if os.path.isabs(filename):
+            return filename
+        path_to_try = os.path.abspath(filename)
+        if os.path.exists(path_to_try):
+            return path_to_try
+        if sys.platform == 'linux' or 'bsd' in sys.platform.lower():
+            paths = [
+                os.environ.get('XDG_DATA_HOME', os.path.expanduser('~/.local/share')),
+            ] + os.environ.get('XDG_DATA_DIRS', '/usr/local/share:/usr/share').split(':')
+            for path in paths:
+                path_to_try = os.path.join(path, 'meson', 'cross', filename)
+                if os.path.exists(path_to_try):
+                    return path_to_try
+            raise MesonException('Cannot find specified cross file: ' + filename)
+
+        raise MesonException('Cannot find specified cross file: ' + filename)
+
     def sanitize_prefix(self, prefix):
         if not os.path.isabs(prefix):
             raise MesonException('prefix value {!r} must be an absolute path'
@@ -205,6 +247,10 @@ class CoreData:
             # string is of type 'C:\' because 'C:' is not an absolute path.
             if len(prefix) == 3 and prefix[1] == ':':
                 pass
+            # If prefix is a single character, preserve it since it is
+            # the root directory.
+            elif len(prefix) == 1:
+                pass
             else:
                 prefix = prefix[:-1]
         return prefix
@@ -245,7 +291,7 @@ class CoreData:
                 value = self.sanitize_dir_option_value(options.prefix, key, value)
                 setattr(options, key, value)
             else:
-                value = get_builtin_option_default(key)
+                value = get_builtin_option_default(key, prefix=options.prefix)
             args = [key] + builtin_options[key][1:-1] + [value]
             self.builtins[key] = builtin_options[key][0](*args)
 
@@ -321,11 +367,19 @@ def get_builtin_option_description(optname):
     else:
         raise RuntimeError('Tried to get the description for an unknown builtin option \'%s\'.' % optname)
 
-def get_builtin_option_default(optname):
+def get_builtin_option_default(optname, prefix='', noneIfSuppress=False):
     if is_builtin_option(optname):
         o = builtin_options[optname]
         if o[0] == UserComboOption:
             return o[3]
+        if optname in builtin_dir_noprefix_options:
+            if noneIfSuppress:
+                # Return None if argparse defaulting should be suppressed for
+                # this option (so we can determine the default later based on
+                # prefix)
+                return None
+            elif prefix in builtin_dir_noprefix_options[optname]:
+                return builtin_dir_noprefix_options[optname][prefix]
         return o[2]
     else:
         raise RuntimeError('Tried to get the default value for an unknown builtin option \'%s\'.' % optname)
@@ -344,15 +398,6 @@ builtin_options = {
     'mandir':     [UserStringOption, 'Manual page directory.', 'share/man'],
     'infodir':    [UserStringOption, 'Info page directory.', 'share/info'],
     'localedir':  [UserStringOption, 'Locale data directory.', 'share/locale'],
-    # sysconfdir, localstatedir and sharedstatedir are a bit special. These defaults to ${prefix}/etc,
-    # ${prefix}/var and ${prefix}/com but nobody uses that. Instead they always set it
-    # manually to /etc, /var and /var/lib. This default values is thus pointless and not really used
-    # but we set it to this for consistency with other systems.
-    #
-    # Projects installing to sysconfdir, localstatedir or sharedstatedir probably want
-    # to set the following in project():
-    #
-    # default_options : ['sysconfdir=/etc', 'localstatedir=/var', 'sharedstatedir=/var/lib']
     'sysconfdir':      [UserStringOption, 'Sysconf data directory.', 'etc'],
     'localstatedir':   [UserStringOption, 'Localstate data directory.', 'var'],
     'sharedstatedir':  [UserStringOption, 'Architecture-independent data directory.', 'com'],
@@ -365,8 +410,13 @@ builtin_options = {
     'errorlogs':       [UserBooleanOption, "Whether to print the logs from failing tests.", True],
 }
 
-# Installation directories that can reside in a path outside of the prefix
-builtin_dir_noprefix_options = {'sysconfdir', 'localstatedir', 'sharedstatedir'}
+# Special prefix-dependent defaults for installation directories that reside in
+# a path outside of the prefix in FHS and common usage.
+builtin_dir_noprefix_options = {
+    'sysconfdir':     {'/usr': '/etc'},
+    'localstatedir':  {'/usr': '/var',     '/usr/local': '/var/local'},
+    'sharedstatedir': {'/usr': '/var/lib', '/usr/local': '/var/local/lib'},
+}
 
 forbidden_target_names = {'clean': None,
                           'clean-ctlist': None,
