@@ -1,4 +1,4 @@
-﻿# Copyright 2013-2017 The Meson development team
+# Copyright 2013-2017 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,13 +21,18 @@ import shlex
 import shutil
 import sysconfig
 
+from pathlib import Path
+
 from .. import mlog
 from .. import mesonlib
 from ..mesonlib import Popen_safe, extract_as_list
 from ..environment import detect_cpu_family
 
-from .base import DependencyException, DependencyMethods
-from .base import ExternalDependency, ExternalProgram, ExtraFrameworkDependency, PkgConfigDependency
+from .base import (
+    DependencyException, DependencyMethods, ExternalDependency,
+    ExternalProgram, ExtraFrameworkDependency, PkgConfigDependency,
+    ConfigToolDependency,
+)
 
 # On windows 3 directory layouts are supported:
 # * The default layout (versioned) installed:
@@ -66,6 +71,11 @@ class BoostDependency(ExternalDependency):
         self.is_multithreading = threading == "multi"
 
         self.requested_modules = self.get_requested(kwargs)
+        invalid_modules = [c for c in self.requested_modules if 'boost_' + c not in BOOST_LIBS]
+        if invalid_modules:
+            mlog.warning('Invalid Boost modules: ' + ', '.join(invalid_modules))
+            self.log_fail()
+            return
 
         self.boost_root = None
         self.boost_roots = []
@@ -198,8 +208,6 @@ class BoostDependency(ExternalDependency):
         for c in candidates:
             if not isinstance(c, str):
                 raise DependencyException('Boost module argument is not a string.')
-            if 'boost_' + c not in BOOST_LIBS:
-                raise DependencyException('Dependency {} not found. It is not a valid boost library.'.format(c))
         return candidates
 
     def validate_requested(self):
@@ -231,17 +239,18 @@ class BoostDependency(ExternalDependency):
 
     def detect_lib_modules_win(self):
         arch = detect_cpu_family(self.env.coredata.compilers)
-        compiler_ts = self.env.detect_cpp_compiler(self.want_cross).get_toolset_version().split('.')
+        comp_ts_version = self.env.detect_cpp_compiler(self.want_cross).get_toolset_version()
+        compiler_ts = comp_ts_version.split('.')
         compiler = 'vc{}{}'.format(compiler_ts[0], compiler_ts[1])
         if not self.libdir:
-            # The libdirs in the distributed binaries
+            # The libdirs in the distributed binaries (from sf)
             if arch == 'x86':
-                gl = 'lib32*'
+                lib_sf = 'lib32-msvc-{}'.format(comp_ts_version)
             elif arch == 'x86_64':
-                gl = 'lib64*'
+                lib_sf = 'lib64-msvc-{}'.format(comp_ts_version)
             else:
                 # Does anyone do Boost cross-compiling to other archs on Windows?
-                gl = None
+                lib_sf = None
             if self.boost_root:
                 roots = [self.boost_root]
             else:
@@ -252,11 +261,10 @@ class BoostDependency(ExternalDependency):
                 if os.path.isdir(libdir):
                     self.libdir = libdir
                     break
-                if gl:
-                    tmp = glob.glob(os.path.join(root, gl))
-                    if len(tmp) > 0:
-                        # FIXME: Should pick the correct version
-                        self.libdir = tmp[0]
+                if lib_sf:
+                    full_path = os.path.join(root, lib_sf)
+                    if os.path.isdir(full_path):
+                        self.libdir = full_path
                         break
 
         if not self.libdir:
@@ -359,11 +367,12 @@ class BoostDependency(ExternalDependency):
             args.append('-L' + self.libdir)
         for lib in self.requested_modules:
             # The compiler's library detector is the most reliable so use that first.
-            default_detect = self.compiler.find_library('boost_' + lib, self.env, [])
+            boost_lib = 'boost_' + lib
+            default_detect = self.compiler.find_library(boost_lib, self.env, [])
             if default_detect is not None:
                 args += default_detect
-            elif lib in self.lib_modules:
-                linkcmd = '-l' + lib
+            elif boost_lib in self.lib_modules:
+                linkcmd = '-l' + boost_lib
                 args.append(linkcmd)
         return args
 
@@ -401,7 +410,7 @@ class MPIDependency(ExternalDependency):
 
         for pkg in pkgconfig_files:
             try:
-                pkgdep = PkgConfigDependency(pkg, environment, kwargs)
+                pkgdep = PkgConfigDependency(pkg, environment, kwargs, language=self.language)
                 if pkgdep.found():
                     self.compile_args = pkgdep.get_compile_args()
                     self.link_args = pkgdep.get_link_args()
@@ -596,17 +605,21 @@ class Python3Dependency(ExternalDependency):
     def __init__(self, environment, kwargs):
         super().__init__('python3', environment, None, kwargs)
         self.name = 'python3'
+        self.static = kwargs.get('static', False)
         # We can only be sure that it is Python 3 at this point
         self.version = '3'
+        self.pkgdep = None
         if DependencyMethods.PKGCONFIG in self.methods:
             try:
-                pkgdep = PkgConfigDependency('python3', environment, kwargs)
-                if pkgdep.found():
-                    self.compile_args = pkgdep.get_compile_args()
-                    self.link_args = pkgdep.get_link_args()
-                    self.version = pkgdep.get_version()
+                self.pkgdep = PkgConfigDependency('python3', environment, kwargs)
+                if self.pkgdep.found():
+                    self.compile_args = self.pkgdep.get_compile_args()
+                    self.link_args = self.pkgdep.get_link_args()
+                    self.version = self.pkgdep.get_version()
                     self.is_found = True
                     return
+                else:
+                    self.pkgdep = None
             except Exception:
                 pass
         if not self.is_found:
@@ -615,8 +628,10 @@ class Python3Dependency(ExternalDependency):
             elif mesonlib.is_osx() and DependencyMethods.EXTRAFRAMEWORK in self.methods:
                 # In OSX the Python 3 framework does not have a version
                 # number in its name.
-                fw = ExtraFrameworkDependency('python', False, None, self.env,
-                                              self.language, kwargs)
+                # There is a python in /System/Library/Frameworks, but that's
+                # python 2, Python 3 will always bin in /Library
+                fw = ExtraFrameworkDependency(
+                    'python', False, '/Library/Frameworks', self.env, self.language, kwargs)
                 if fw.found():
                     self.compile_args = fw.get_compile_args()
                     self.link_args = fw.get_link_args()
@@ -626,12 +641,55 @@ class Python3Dependency(ExternalDependency):
         else:
             mlog.log('Dependency', mlog.bold(self.name), 'found:', mlog.red('NO'))
 
+    @staticmethod
+    def get_windows_python_arch():
+        pyplat = sysconfig.get_platform()
+        if pyplat == 'mingw':
+            pycc = sysconfig.get_config_var('CC')
+            if pycc.startswith('x86_64'):
+                return '64'
+            elif pycc.startswith(('i686', 'i386')):
+                return '32'
+            else:
+                mlog.log('MinGW Python built with unknown CC {!r}, please file'
+                         'a bug'.format(pycc))
+                return None
+        elif pyplat == 'win32':
+            return '32'
+        elif pyplat in ('win64', 'win-amd64'):
+            return '64'
+        mlog.log('Unknown Windows Python platform {!r}'.format(pyplat))
+        return None
+
+    def get_windows_link_args(self):
+        pyplat = sysconfig.get_platform()
+        if pyplat.startswith('win'):
+            vernum = sysconfig.get_config_var('py_version_nodot')
+            if self.static:
+                libname = 'libpython{}.a'.format(vernum)
+            else:
+                libname = 'python{}.lib'.format(vernum)
+            lib = Path(sysconfig.get_config_var('base')) / 'libs' / libname
+        elif pyplat == 'mingw':
+            if self.static:
+                libname = sysconfig.get_config_var('LIBRARY')
+            else:
+                libname = sysconfig.get_config_var('LDLIBRARY')
+            lib = Path(sysconfig.get_config_var('LIBDIR')) / libname
+        if not lib.exists():
+            mlog.log('Could not find Python3 library {!r}'.format(str(lib)))
+            return None
+        return [str(lib)]
+
     def _find_libpy3_windows(self, env):
         '''
         Find python3 libraries on Windows and also verify that the arch matches
         what we are building for.
         '''
-        pyarch = sysconfig.get_platform()
+        pyarch = self.get_windows_python_arch()
+        if pyarch is None:
+            self.is_found = False
+            return
         arch = detect_cpu_family(env.coredata.compilers)
         if arch == 'x86':
             arch = '32'
@@ -644,22 +702,24 @@ class Python3Dependency(ExternalDependency):
             self.is_found = False
             return
         # Pyarch ends in '32' or '64'
-        if arch != pyarch[-2:]:
-            mlog.log('Need', mlog.bold(self.name),
-                     'for {}-bit, but found {}-bit'.format(arch, pyarch[-2:]))
+        if arch != pyarch:
+            mlog.log('Need', mlog.bold(self.name), 'for {}-bit, but '
+                     'found {}-bit'.format(arch, pyarch))
             self.is_found = False
             return
+        # This can fail if the library is not found
+        largs = self.get_windows_link_args()
+        if largs is None:
+            self.is_found = False
+            return
+        self.link_args = largs
+        # Compile args
         inc = sysconfig.get_path('include')
         platinc = sysconfig.get_path('platinclude')
         self.compile_args = ['-I' + inc]
         if inc != platinc:
             self.compile_args.append('-I' + platinc)
-        # Nothing exposes this directly that I coulf find
-        basedir = sysconfig.get_config_var('base')
-        vernum = sysconfig.get_config_var('py_version_nodot')
-        self.link_args = ['-L{}/libs'.format(basedir),
-                          '-lpython{}'.format(vernum)]
-        self.version = sysconfig.get_config_var('py_version_short')
+        self.version = sysconfig.get_config_var('py_version')
         self.is_found = True
 
     def get_methods(self):
@@ -670,13 +730,19 @@ class Python3Dependency(ExternalDependency):
         else:
             return [DependencyMethods.PKGCONFIG]
 
+    def get_pkgconfig_variable(self, variable_name, kwargs):
+        if self.pkgdep:
+            return self.pkgdep.get_pkgconfig_variable(variable_name, kwargs)
+        else:
+            return super().get_pkgconfig_variable(variable_name, kwargs)
+
 
 class PcapDependency(ExternalDependency):
     def __init__(self, environment, kwargs):
         super().__init__('pcap', environment, None, kwargs)
+        kwargs['required'] = False
         if DependencyMethods.PKGCONFIG in self.methods:
             try:
-                kwargs['required'] = False
                 pcdep = PkgConfigDependency('pcap', environment, kwargs)
                 if pcdep.found():
                     self.type_name = 'pkgconfig'
@@ -687,25 +753,26 @@ class PcapDependency(ExternalDependency):
                     return
             except Exception as e:
                 mlog.debug('Pcap not found via pkgconfig. Trying next, error was:', str(e))
-        if DependencyMethods.PCAPCONFIG in self.methods:
-            pcapconf = shutil.which('pcap-config')
-            if pcapconf:
-                stdo = Popen_safe(['pcap-config', '--cflags'])[1]
-                self.compile_args = stdo.strip().split()
-                stdo = Popen_safe(['pcap-config', '--libs'])[1]
-                self.link_args = stdo.strip().split()
-                self.version = self.get_pcap_lib_version()
-                self.is_found = True
-                mlog.log('Dependency', mlog.bold('pcap'), 'found:',
-                         mlog.green('YES'), '(%s)' % pcapconf)
-                return
-            mlog.debug('Could not find pcap-config binary, trying next.')
+        if DependencyMethods.CONFIG_TOOL in self.methods:
+            try:
+                ctdep = ConfigToolDependency.factory(
+                    'pcap', environment, None, kwargs, ['pcap-config'], 'pcap-config')
+                if ctdep.found():
+                    self.config = ctdep.config
+                    self.type_name = 'config-tool'
+                    self.compile_args = ctdep.get_config_value(['--cflags'], 'compile_args')
+                    self.link_args = ctdep.get_config_value(['--libs'], 'link_args')
+                    self.version = self.get_pcap_lib_version()
+                    self.is_found = True
+                    return
+            except Exception as e:
+                mlog.debug('Pcap not found via pcap-config. Trying next, error was:', str(e))
 
     def get_methods(self):
         if mesonlib.is_osx():
-            return [DependencyMethods.PKGCONFIG, DependencyMethods.PCAPCONFIG, DependencyMethods.EXTRAFRAMEWORK]
+            return [DependencyMethods.PKGCONFIG, DependencyMethods.CONFIG_TOOL, DependencyMethods.EXTRAFRAMEWORK]
         else:
-            return [DependencyMethods.PKGCONFIG, DependencyMethods.PCAPCONFIG]
+            return [DependencyMethods.PKGCONFIG, DependencyMethods.CONFIG_TOOL]
 
     def get_pcap_lib_version(self):
         return self.compiler.get_return_value('pcap_lib_version', 'string',
@@ -715,9 +782,9 @@ class PcapDependency(ExternalDependency):
 class CupsDependency(ExternalDependency):
     def __init__(self, environment, kwargs):
         super().__init__('cups', environment, None, kwargs)
+        kwargs['required'] = False
         if DependencyMethods.PKGCONFIG in self.methods:
             try:
-                kwargs['required'] = False
                 pcdep = PkgConfigDependency('cups', environment, kwargs)
                 if pcdep.found():
                     self.type_name = 'pkgconfig'
@@ -728,20 +795,20 @@ class CupsDependency(ExternalDependency):
                     return
             except Exception as e:
                 mlog.debug('cups not found via pkgconfig. Trying next, error was:', str(e))
-        if DependencyMethods.CUPSCONFIG in self.methods:
-            cupsconf = shutil.which('cups-config')
-            if cupsconf:
-                stdo = Popen_safe(['cups-config', '--cflags'])[1]
-                self.compile_args = stdo.strip().split()
-                stdo = Popen_safe(['cups-config', '--libs'])[1]
-                self.link_args = stdo.strip().split()
-                stdo = Popen_safe(['cups-config', '--version'])[1]
-                self.version = stdo.strip().split()
-                self.is_found = True
-                mlog.log('Dependency', mlog.bold('cups'), 'found:',
-                         mlog.green('YES'), '(%s)' % cupsconf)
-                return
-            mlog.debug('Could not find cups-config binary, trying next.')
+        if DependencyMethods.CONFIG_TOOL in self.methods:
+            try:
+                ctdep = ConfigToolDependency.factory(
+                    'cups', environment, None, kwargs, ['cups-config'], 'cups-config')
+                if ctdep.found():
+                    self.config = ctdep.config
+                    self.type_name = 'config-tool'
+                    self.version = ctdep.version
+                    self.compile_args = ctdep.get_config_value(['--cflags'], 'compile_args')
+                    self.link_args = ctdep.get_config_value(['--libs'], 'link_args')
+                    self.is_found = True
+                    return
+            except Exception as e:
+                mlog.debug('cups not found via cups-config. Trying next, error was:', str(e))
         if DependencyMethods.EXTRAFRAMEWORK in self.methods:
             if mesonlib.is_osx():
                 fwdep = ExtraFrameworkDependency('cups', False, None, self.env,
@@ -756,9 +823,48 @@ class CupsDependency(ExternalDependency):
 
     def get_methods(self):
         if mesonlib.is_osx():
-            return [DependencyMethods.PKGCONFIG, DependencyMethods.CUPSCONFIG, DependencyMethods.EXTRAFRAMEWORK]
+            return [DependencyMethods.PKGCONFIG, DependencyMethods.CONFIG_TOOL, DependencyMethods.EXTRAFRAMEWORK]
         else:
-            return [DependencyMethods.PKGCONFIG, DependencyMethods.CUPSCONFIG]
+            return [DependencyMethods.PKGCONFIG, DependencyMethods.CONFIG_TOOL]
+
+
+class LibWmfDependency(ExternalDependency):
+    def __init__(self, environment, kwargs):
+        super().__init__('libwmf', environment, None, kwargs)
+        if DependencyMethods.PKGCONFIG in self.methods:
+            try:
+                kwargs['required'] = False
+                pcdep = PkgConfigDependency('libwmf', environment, kwargs)
+                if pcdep.found():
+                    self.type_name = 'pkgconfig'
+                    self.is_found = True
+                    self.compile_args = pcdep.get_compile_args()
+                    self.link_args = pcdep.get_link_args()
+                    self.version = pcdep.get_version()
+                    return
+            except Exception as e:
+                mlog.debug('LibWmf not found via pkgconfig. Trying next, error was:', str(e))
+        if DependencyMethods.CONFIG_TOOL in self.methods:
+            try:
+                ctdep = ConfigToolDependency.factory(
+                    'libwmf', environment, None, kwargs, ['libwmf-config'], 'libwmf-config')
+                if ctdep.found():
+                    self.config = ctdep.config
+                    self.type_name = 'config-too'
+                    self.version = ctdep.version
+                    self.compile_args = ctdep.get_config_value(['--cflags'], 'compile_args')
+                    self.link_args = ctdep.get_config_value(['--libs'], 'link_args')
+                    self.is_found = True
+                    return
+            except Exception as e:
+                mlog.debug('cups not found via libwmf-config. Trying next, error was:', str(e))
+
+    def get_methods(self):
+        if mesonlib.is_osx():
+            return [DependencyMethods.PKGCONFIG, DependencyMethods.CONFIG_TOOL, DependencyMethods.EXTRAFRAMEWORK]
+        else:
+            return [DependencyMethods.PKGCONFIG, DependencyMethods.CONFIG_TOOL]
+
 
 # Generated with boost_names.py
 BOOST_LIBS = [
@@ -814,6 +920,11 @@ BOOST_LIBS = [
     'boost_math_c99f',
     'boost_math_c99l',
     'boost_mpi',
+    'boost_program_options',
+    'boost_python',
+    'boost_python3',
+    'boost_numpy',
+    'boost_numpy3',
     'boost_random',
     'boost_regex',
     'boost_serialization',
