@@ -17,7 +17,7 @@ import pickle
 import argparse
 from . import coredata, mesonlib
 
-parser = argparse.ArgumentParser()
+parser = argparse.ArgumentParser(prog='meson configure')
 
 parser.add_argument('-D', action='append', default=[], dest='sets',
                     help='Set an option to the given value.')
@@ -51,7 +51,7 @@ class Conf:
         with open(self.coredata_file, 'wb') as f:
             pickle.dump(self.coredata, f)
         # We don't write the build file because any changes to it
-        # are erased when Meson is executed the next time, i.e. whne
+        # are erased when Meson is executed the next time, i.e. when
         # Ninja is run.
 
     def print_aligned(self, arr):
@@ -184,7 +184,7 @@ class Conf:
             coarr = []
             for k in okeys:
                 o = self.coredata.base_options[k]
-                coarr.append({'name': k, 'descr': o.description, 'value': o.value, 'choices': ''})
+                coarr.append({'name': k, 'descr': o.description, 'value': o.value, 'choices': o.choices})
             self.print_aligned(coarr)
         print('')
         print('Compiler arguments:')
