@@ -58,7 +58,7 @@ def determine_worker_count():
             num_workers = 1
     return num_workers
 
-parser = argparse.ArgumentParser()
+parser = argparse.ArgumentParser(prog='meson test')
 parser.add_argument('--repeat', default=1, dest='repeat', type=int,
                     help='Number of times to run the tests.')
 parser.add_argument('--no-rebuild', default=False, action='store_true',
@@ -267,7 +267,13 @@ class TestHarness:
                 if is_windows():
                     subprocess.call(['taskkill', '/F', '/T', '/PID', str(p.pid)])
                 else:
-                    os.killpg(os.getpgid(p.pid), signal.SIGKILL)
+                    try:
+                        os.killpg(os.getpgid(p.pid), signal.SIGKILL)
+                    except ProcessLookupError:
+                        # Sometimes (e.g. with Wine) this happens.
+                        # There's nothing we can do (maybe the process
+                        # already died) so carry on.
+                        pass
                 (stdo, stde) = p.communicate()
             endtime = time.time()
             duration = endtime - starttime
@@ -476,35 +482,41 @@ TIMEOUT: %4d
         numlen = len('%d' % len(tests))
         self.open_log_files()
         wrap = self.get_wrapper()
+        startdir = os.getcwd()
+        if self.options.wd:
+            os.chdir(self.options.wd)
 
-        for _ in range(self.options.repeat):
-            for i, test in enumerate(tests):
-                visible_name = self.get_pretty_suite(test)
+        try:
+            for _ in range(self.options.repeat):
+                for i, test in enumerate(tests):
+                    visible_name = self.get_pretty_suite(test)
 
-                if self.options.gdb:
-                    test.timeout = None
+                    if self.options.gdb:
+                        test.timeout = None
 
-                if not test.is_parallel or self.options.gdb:
-                    self.drain_futures(futures)
-                    futures = []
-                    res = self.run_single_test(wrap, test)
-                    self.print_stats(numlen, tests, visible_name, res, i)
-                else:
-                    if not executor:
-                        executor = conc.ThreadPoolExecutor(max_workers=self.options.num_processes)
-                    f = executor.submit(self.run_single_test, wrap, test)
-                    futures.append((f, numlen, tests, visible_name, i))
+                    if not test.is_parallel or self.options.gdb:
+                        self.drain_futures(futures)
+                        futures = []
+                        res = self.run_single_test(wrap, test)
+                        self.print_stats(numlen, tests, visible_name, res, i)
+                    else:
+                        if not executor:
+                            executor = conc.ThreadPoolExecutor(max_workers=self.options.num_processes)
+                        f = executor.submit(self.run_single_test, wrap, test)
+                        futures.append((f, numlen, tests, visible_name, i))
+                    if self.options.repeat > 1 and self.fail_count:
+                        break
                 if self.options.repeat > 1 and self.fail_count:
                     break
-            if self.options.repeat > 1 and self.fail_count:
-                break
 
-        self.drain_futures(futures)
-        self.print_summary()
-        self.print_collected_logs()
+            self.drain_futures(futures)
+            self.print_summary()
+            self.print_collected_logs()
 
-        if self.logfilename:
-            print('Full log written to %s' % self.logfilename)
+            if self.logfilename:
+                print('Full log written to %s' % self.logfilename)
+        finally:
+            os.chdir(startdir)
 
     def drain_futures(self, futures):
         for i in futures:
@@ -619,6 +631,6 @@ def run(args):
             return th.doit()
         return th.run_special()
     except TestException as e:
-        print('Mesontest encountered an error:\n')
+        print('Meson test encountered an error:\n')
         print(e)
         return 1
